@@ -13,6 +13,8 @@
 
 #include "cmSystemTools.h"
 #include "cmMakefile.h"
+#include "cmState.h"
+#include "cmLocalGenerator.h"
 
 #include "cmCommandArgumentLexer.h"
 
@@ -90,7 +92,8 @@ char* cmCommandArgumentParserHelper::ExpandSpecialVariable(const char* key,
     }
   if ( strcmp(key, "CACHE") == 0 )
     {
-    if(const char* c = this->Makefile->GetCacheManager()->GetCacheValue(var))
+    if(const char* c = this->Makefile->GetState()
+                           ->GetInitializedCacheValue(var))
       {
       if(this->EscapeQuotes)
         {
@@ -103,7 +106,7 @@ char* cmCommandArgumentParserHelper::ExpandSpecialVariable(const char* key,
       }
     return this->EmptyVariable;
     }
-  cmOStringStream e;
+  std::ostringstream e;
   e << "Syntax $" << key << "{} is not supported.  "
     << "Only ${}, $ENV{}, and $CACHE{} are allowed.";
   this->SetError(e.str());
@@ -118,7 +121,7 @@ char* cmCommandArgumentParserHelper::ExpandVariable(const char* var)
     }
   if(this->FileLine >= 0 && strcmp(var, "CMAKE_CURRENT_LIST_LINE") == 0)
     {
-    cmOStringStream ostr;
+    std::ostringstream ostr;
     ostr << this->FileLine;
     return this->AddString(ostr.str());
     }
@@ -136,15 +139,15 @@ char* cmCommandArgumentParserHelper::ExpandVariable(const char* var)
           cmSystemTools::IsSubDirectory(this->FileName,
                                      this->Makefile->GetHomeOutputDirectory()))
         {
-        cmOStringStream msg;
-        cmListFileBacktrace bt(this->Makefile->GetLocalGenerator());
+        std::ostringstream msg;
         cmListFileContext lfc;
-        lfc.FilePath = this->FileName;
+        lfc.FilePath = this->Makefile->GetLocalGenerator()
+            ->Convert(this->FileName, cmLocalGenerator::HOME);
+
         lfc.Line = this->FileLine;
-        bt.push_back(lfc);
         msg << "uninitialized variable \'" << var << "\'";
         this->Makefile->GetCMakeInstance()->IssueMessage(cmake::AUTHOR_WARNING,
-                                                        msg.str(), bt);
+                                                        msg.str(), lfc);
         }
       }
     return 0;
@@ -253,7 +256,7 @@ bool cmCommandArgumentParserHelper::HandleEscapeSymbol
     break;
   default:
     {
-    cmOStringStream e;
+    std::ostringstream e;
     e << "Invalid escape sequence \\" << symbol;
     this->SetError(e.str());
     }
@@ -335,7 +338,7 @@ int cmCommandArgumentParserHelper::LexInput(char* buf, int maxlen)
 void cmCommandArgumentParserHelper::Error(const char* str)
 {
   unsigned long pos = static_cast<unsigned long>(this->InputBufferPos);
-  cmOStringStream ostr;
+  std::ostringstream ostr;
   ostr << str << " (" << pos << ")";
   this->SetError(ostr.str());
 }
