@@ -21,16 +21,16 @@ class cmExportTryCompileFileGenerator: public cmExportFileGenerator
 {
 public:
   /** Set the list of targets to export.  */
-  void SetExports(const std::vector<cmTarget*> &exports)
+  void SetExports(const std::vector<cmTarget const*> &exports)
     { this->Exports = exports; }
-  void SetConfig(const char *config) { this->Config = config; }
+  void SetConfig(const std::string& config) { this->Config = config; }
 protected:
 
   // Implement virtual methods from the superclass.
   virtual bool GenerateMainFile(std::ostream& os);
 
   virtual void GenerateImportTargetsConfig(std::ostream&,
-                                           const char*,
+                                           const std::string&,
                                            std::string const&,
                             std::vector<std::string>&) {}
   virtual void HandleMissingTarget(std::string&,
@@ -39,19 +39,19 @@ protected:
                                    cmTarget*,
                                    cmTarget*) {}
 
-  void PopulateProperties(cmTarget* target,
+  void PopulateProperties(cmTarget const* target,
                           ImportPropertyMap& properties,
-                          std::set<cmTarget*> &emitted);
+                          std::set<cmTarget const*> &emitted);
 
   std::string InstallNameDir(cmTarget* target,
                              const std::string& config);
 private:
-  std::string FindTargets(const char *prop, cmTarget *tgt,
-                   std::set<cmTarget*> &emitted);
+  std::string FindTargets(const std::string& prop, cmTarget const* tgt,
+                   std::set<cmTarget const*> &emitted);
 
 
-  std::vector<cmTarget*> Exports;
-  const char *Config;
+  std::vector<cmTarget const*> Exports;
+  std::string Config;
 };
 
 #endif
