@@ -12,6 +12,7 @@
 #include "cmInstallTargetGenerator.h"
 
 #include "cmComputeLinkInformation.h"
+#include "cmGeneratorExpression.h"
 #include "cmGlobalGenerator.h"
 #include "cmLocalGenerator.h"
 #include "cmMakefile.h"
@@ -68,7 +69,7 @@ void cmInstallTargetGenerator::GenerateScriptForConfig(std::ostream& os,
   std::string fromDirConfig;
   if(this->Target->NeedRelinkBeforeInstall(config))
     {
-    fromDirConfig = this->Target->GetMakefile()->GetStartOutputDirectory();
+    fromDirConfig = this->Target->GetMakefile()->GetCurrentBinaryDirectory();
     fromDirConfig += cmake::GetCMakeFilesDirectory();
     fromDirConfig += "/CMakeRelink.dir/";
     }
@@ -77,7 +78,8 @@ void cmInstallTargetGenerator::GenerateScriptForConfig(std::ostream& os,
     fromDirConfig = this->Target->GetDirectory(config, this->ImportLibrary);
     fromDirConfig += "/";
     }
-  std::string toDir = this->GetInstallDestination();
+  std::string toDir =
+    this->ConvertToAbsoluteDestination(this->GetDestination(config));
   toDir += "/";
 
   // Compute the list of files to install for this target.
@@ -322,8 +324,8 @@ void cmInstallTargetGenerator::GenerateScriptForConfig(std::ostream& os,
   const char* no_dir_permissions = 0;
   const char* no_rename = 0;
   bool optional = this->Optional || this->ImportLibrary;
-  this->AddInstallRule(os, type, filesFrom,
-                       optional,
+  this->AddInstallRule(os, this->GetDestination(config),
+                       type, filesFrom, optional,
                        this->FilePermissions.c_str(), no_dir_permissions,
                        no_rename, literal_args.c_str(),
                        indent);
@@ -333,6 +335,15 @@ void cmInstallTargetGenerator::GenerateScriptForConfig(std::ostream& os,
                  &cmInstallTargetGenerator::PostReplacementTweaks);
 }
 
+//----------------------------------------------------------------------------
+std::string
+cmInstallTargetGenerator::GetDestination(std::string const& config) const
+{
+  cmGeneratorExpression ge;
+  return ge.Parse(this->Destination)
+    ->Evaluate(this->Target->GetMakefile(), config);
+}
+
 //----------------------------------------------------------------------------
 std::string
 cmInstallTargetGenerator::GetInstallFilename(const std::string& config) const
@@ -717,8 +728,7 @@ cmInstallTargetGenerator
           i != oldRuntimeDirs.end(); ++i)
         {
         std::string runpath =
-          mf->GetLocalGenerator()->
-          GetGlobalGenerator()->ExpandCFGIntDir(*i, config);
+          mf->GetGlobalGenerator()->ExpandCFGIntDir(*i, config);
 
         if(runpaths.find(runpath) == runpaths.end())
           {
@@ -734,8 +744,7 @@ cmInstallTargetGenerator
           i != newRuntimeDirs.end(); ++i)
         {
         std::string runpath =
-          mf->GetLocalGenerator()->
-          GetGlobalGenerator()->ExpandCFGIntDir(*i, config);
+          mf->GetGlobalGenerator()->ExpandCFGIntDir(*i, config);
 
         if(runpaths.find(runpath) == runpaths.end())
           {
