@@ -26,62 +26,46 @@ bool cmSetTestsPropertiesCommand
 
   // first collect up the list of files
   std::vector<std::string> propertyPairs;
-  bool doingFiles = true;
   int numFiles = 0;
   std::vector<std::string>::const_iterator j;
   for(j= args.begin(); j != args.end();++j)
     {
     if(*j == "PROPERTIES")
       {
-      doingFiles = false;
       // now loop through the rest of the arguments, new style
       ++j;
-      while (j != args.end())
+      if (std::distance(j, args.end()) % 2 != 0)
         {
-        propertyPairs.push_back(*j);
-        ++j;
-        if(j == args.end())
-          {
-          this->SetError("called with incorrect number of arguments.");
-          return false;
-          }
-        propertyPairs.push_back(*j);
-        ++j;
+        this->SetError("called with incorrect number of arguments.");
+        return false;
         }
-      // break out of the loop because j is already == end
+      propertyPairs.insert(propertyPairs.end(), j, args.end());
       break;
       }
-    else if (doingFiles)
-      {
-      numFiles++;
-      }
     else
       {
-      this->SetError("called with illegal arguments, maybe "
-                     "missing a PROPERTIES specifier?");
-      return false;
+      numFiles++;
       }
     }
-  if(propertyPairs.size() == 0)
+  if(propertyPairs.empty())
     {
     this->SetError("called with illegal arguments, maybe "
                    "missing a PROPERTIES specifier?");
     return false;
     }
 
-
   // now loop over all the targets
   int i;
   for(i = 0; i < numFiles; ++i)
-    {   
+    {
     std::string errors;
-    bool ret = 
-      cmSetTestsPropertiesCommand::SetOneTest(args[i].c_str(), 
+    bool ret =
+      cmSetTestsPropertiesCommand::SetOneTest(args[i],
                                               propertyPairs,
                                               this->Makefile, errors);
     if (!ret)
       {
-      this->SetError(errors.c_str());
+      this->SetError(errors);
       return ret;
       }
     }
@@ -91,7 +75,7 @@ bool cmSetTestsPropertiesCommand
 
 
 bool cmSetTestsPropertiesCommand
-::SetOneTest(const char *tname, 
+::SetOneTest(const std::string& tname,
              std::vector<std::string> &propertyPairs,
              cmMakefile *mf, std::string &errors)
 {
@@ -101,16 +85,16 @@ bool cmSetTestsPropertiesCommand
     unsigned int k;
     for (k = 0; k < propertyPairs.size(); k = k + 2)
       {
-      test->SetProperty(propertyPairs[k].c_str(),
+      test->SetProperty(propertyPairs[k],
                         propertyPairs[k+1].c_str());
       }
     }
   else
-    { 
+    {
     errors = "Can not find test to add properties to: ";
     errors += tname;
     return false;
-    } 
+    }
 
   return true;
 }
